@@ -71,17 +71,14 @@ public:
   // Files included by the preprocessor and their hashes.
   std::unordered_map<std::string, Digest> included_files;
 
-  // Uses absolute path for some include files.
-  bool has_absolute_include_headers = false;
-
   // Have we tried and failed to get colored diagnostics?
   bool diagnostics_color_failed = false;
 
   // The name of the temporary preprocessed file.
   std::string i_tmpfile;
 
-  // The name of the cpp stderr file.
-  std::string cpp_stderr;
+  // The preprocessor's stderr output.
+  std::string cpp_stderr_data;
 
   // Headers (or directories with headers) to ignore in manifest mode.
   std::vector<std::string> ignore_header_paths;
